@@ -188,17 +188,14 @@ gboolean bitlbee_io_current_client_write( GIOChannel *source, GIOCondition condi
 	size = strlen( irc->sendbuffer );
 	st = write( irc->fd, irc->sendbuffer, size );
 	
-	if( st <= 0 )
+	if( st == 0 || ( st < 0 && !sockerr_again() ) )
 	{
-		if( sockerr_again() )
-		{
-			return TRUE;
-		}
-		else
-		{
-			irc_free( irc );
-			return FALSE;
-		}
+		irc_free( irc );
+		return FALSE;
+	}
+	else if( st < 0 ) /* && sockerr_again() */
+	{
+		return TRUE;
 	}
 	
 	if( st == size )
