@@ -29,7 +29,7 @@
 #define _GNU_SOURCE /* Stupid GNU :-P */
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "1.0.1"
+#define BITLBEE_VERSION "1.0.2"
 #define VERSION BITLBEE_VERSION
 
 #define MAX_STRING 128
@@ -109,6 +109,7 @@ extern char *CONF_FILE;
 #include "help.h"
 #include "query.h"
 #include "sock.h"
+#include "util.h"
 
 typedef struct global_t {
 	int listen_socket;
@@ -129,10 +130,6 @@ int root_command( irc_t *irc, char *command[] );
 int bitlbee_load( irc_t *irc, const char *password );
 int bitlbee_save( irc_t *irc );
 void bitlbee_shutdown( gpointer data );
-double gettime( void );
-G_MODULE_EXPORT void http_encode( char *s );
-G_MODULE_EXPORT void http_decode( char *s );
-G_MODULE_EXPORT char *strip_newlines(char *source);
 
 extern global_t global;
 
