@@ -487,7 +487,27 @@ int cmd_block( irc_t *irc, char **cmd )
 	struct gaim_connection *gc;
 	account_t *a;
 	
-	if( !cmd[2] )
+	if( !cmd[2] && ( a = account_get( irc, cmd[1] ) ) && a->gc )
+	{
+		char *format;
+		GSList *l;
+		
+		if( strchr( irc->umode, 'b' ) != NULL )
+			format = "%s\t%s";
+		else
+			format = "%-32.32  %-16.16s";
+		
+		irc_usermsg( irc, format, "Handle", "Nickname" );
+		for( l = a->gc->deny; l; l = l->next )
+		{
+			user_t *u = user_findhandle( a->gc, l->data );
+			irc_usermsg( irc, format, l->data, u ? u->nick : "(none)" );
+		}
+		irc_usermsg( irc, "End of list." );
+		
+		return 1;
+	}
+	else if( !cmd[2] )
 	{
 		user_t *u = user_find( irc, cmd[1] );
 		if( !u || !u->gc )
@@ -528,7 +548,27 @@ int cmd_allow( irc_t *irc, char **cmd )
 	struct gaim_connection *gc;
 	account_t *a;
 	
-	if( !cmd[2] )
+	if( !cmd[2] && ( a = account_get( irc, cmd[1] ) ) && a->gc )
+	{
+		char *format;
+		GSList *l;
+		
+		if( strchr( irc->umode, 'b' ) != NULL )
+			format = "%s\t%s";
+		else
+			format = "%-32.32  %-16.16s";
+		
+		irc_usermsg( irc, format, "Handle", "Nickname" );
+		for( l = a->gc->deny; l; l = l->next )
+		{
+			user_t *u = user_findhandle( a->gc, l->data );
+			irc_usermsg( irc, format, l->data, u ? u->nick : "(none)" );
+		}
+		irc_usermsg( irc, "End of list." );
+		
+		return 1;
+	}
+	else if( !cmd[2] )
 	{
 		user_t *u = user_find( irc, cmd[1] );
 		if( !u || !u->gc )
@@ -610,6 +650,9 @@ int cmd_set( irc_t *irc, char **cmd )
 	if( cmd[1] && cmd[2] )
 	{
 		set_setstr( irc, cmd[1], cmd[2] );
+		
+		if( ( strcmp( cmd[2], "=" ) ) == 0 && cmd[3] )
+			irc_usermsg( irc, "Warning: Correct syntax: \002set <variable> <value>\002 (without =)" );
 	}
 	if( cmd[1] ) /* else 'forgotten' on purpose.. Must show new value after changing */
 	{
@@ -709,15 +752,9 @@ int cmd_nick( irc_t *irc, char **cmd )
 	}
 	else
 	{
-		char utf8[1024];
-		
 		irc_usermsg( irc, "Setting your name to `%s'", cmd[2] );
 		
-		if( g_strncasecmp( set_getstr( irc, "charset" ), "none", 4 ) != 0 &&
-		    do_iconv( set_getstr( irc, "charset" ), "UTF-8", cmd[2], utf8, 0, 1024 ) != -1 )
-			a->gc->prpl->set_info( a->gc, utf8 );
-		else
-			a->gc->prpl->set_info( a->gc, cmd[2] );
+		a->gc->prpl->set_info( a->gc, cmd[2] );
 	}
 	
 	return( 1 );
