@@ -165,9 +165,12 @@ void irc_free(irc_t * irc)
 	g_io_channel_unref( irc->io_channel );
 	irc_connection_list = g_slist_remove( irc_connection_list, irc );
 	
-	for (account = irc->accounts; account; account = account->next)
+	for (account = irc->accounts; account; account = account->next) {
 		if (account->gc)
-			signoff(account->gc);
+			account_offline(account->gc);
+		else if (account->reconnect)
+			cancel_auto_reconnect(account);
+	}
 	
 	g_free(irc->sendbuffer);
 	g_free(irc->readbuffer);
@@ -262,19 +265,27 @@ void irc_free(irc_t * irc)
 
 int irc_process( irc_t *irc )
 {
-	char **lines, *temp;	
+	char **lines, *temp, *cs;
 	int i;
 
 	if( irc->readbuffer != NULL ) {
 		lines = irc_tokenize(irc->readbuffer );
 		for( i = 0; *lines[i] != '\0'; i++ ) {
+			char conv[IRC_MAX_LINE+1];
+			
 			if( lines[i+1] == NULL ) {
 				temp = g_strdup( lines[i] );
 				g_free( irc->readbuffer );
 				irc->readbuffer = temp;
 				i++;
 				break;
-			}			
+			}
+			if( ( cs = set_getstr( irc, "charset" ) ) && ( g_strcasecmp( cs, "utf-8" ) != 0 ) )
+			{
+				conv[IRC_MAX_LINE] = 0;
+				if( do_iconv( cs, "UTF-8", lines[i], conv, 0, IRC_MAX_LINE - 2 ) != -1 )
+					lines[i] = conv;
+			}
 			if (!irc_process_line(irc, lines[i])) {
 				g_free( lines );
 				return 0;
@@ -889,16 +900,25 @@ void irc_write( irc_t *irc, char *format, ... )
 void irc_vawrite( irc_t *irc, char *format, va_list params )
 {
 	int size;
-	char line[IRC_MAX_LINE];
-	
+	char line[IRC_MAX_LINE+1], *cs;
+		
 	if( irc->quit )
 		return;
-
-	g_vsnprintf( line, IRC_MAX_LINE - 3, format, params );
-
+	
+	line[IRC_MAX_LINE] = 0;
+	g_vsnprintf( line, IRC_MAX_LINE - 2, format, params );
+	
 	strip_newlines( line );
+	if( ( cs = set_getstr( irc, "charset" ) ) && ( g_strcasecmp( cs, "utf-8" ) != 0 ) )
+	{
+		char conv[IRC_MAX_LINE+1];
+		
+		conv[IRC_MAX_LINE] = 0;
+		if( do_iconv( "UTF-8", cs, line, conv, 0, IRC_MAX_LINE - 2 ) != -1 )
+			strcpy( line, conv );
+	}
 	strcat( line, "\r\n" );
-
+	
 	if( irc->sendbuffer != NULL ) {
 		size = strlen( irc->sendbuffer ) + strlen( line );
 		irc->sendbuffer = g_renew ( char, irc->sendbuffer, size + 1 );
@@ -1119,7 +1139,7 @@ void irc_motd( irc_t *irc )
 			}
 		}
 		irc_reply( irc, 376, ":End of MOTD" );
-		closesocket( fd );
+		close( fd );
 	}
 }
 
@@ -1198,7 +1218,7 @@ void irc_umode_set( irc_t *irc, char *who, char *s )
 int irc_away( irc_t *irc, char *away )
 {
 	user_t *u = user_find( irc, irc->nick );
-	GSList *c = get_connections();
+	account_t *a;
 	
 	if( !u ) return( 0 );
 	
@@ -1225,12 +1245,12 @@ int irc_away( irc_t *irc, char *away )
 		irc_reply( irc, 305, ":Welcome back" );
 	}
 	
-	while( c )
+	for( a = irc->accounts; a; a = a->next )
 	{
-		if( ((struct gaim_connection *)c->data)->flags & OPT_LOGGED_IN )
-			proto_away( c->data, u->away );
-		
-		c = c->next;
+		struct gaim_connection *gc = a->gc;
+ 		
+		if( gc && gc->flags & OPT_LOGGED_IN )
+			proto_away( gc, u->away );
 	}
 	
 	return( 1 );
