@@ -417,7 +417,7 @@ static void gjab_recv(gjconn gjc)
 		XML_Parse(gjc->parser, buf, len, 0);
 		if (jd->die)
 			signoff(GJ_GC(gjc));
-	} else if (len < 0 || errno != EAGAIN) {
+	} else if (len == 0 || (len < 0 && (!sockerr_again() || gjc->ssl))) {
 		STATE_EVT(JCONN_STATE_OFF)
 	}
 }
@@ -1236,9 +1236,7 @@ static void jabber_handleroster(gjconn gjc, xmlnode querynode)
 		x = xmlnode_get_nextsibling(x);
 	}
 
-	x = jutil_presnew(0, NULL, "Online");
-	gjab_send(gjc, x);
-	xmlnode_free(x);
+	account_online(GJ_GC(gjc));
 }
 
 static void jabber_handleauthresp(gjconn gjc, jpacket p)
@@ -1254,7 +1252,6 @@ static void jabber_handleauthresp(gjconn gjc, jpacket p)
 			gjab_auth(gjc);
 		} else {
 			gjab_reqroster(gjc);
-			account_online(GJ_GC(gjc));
 			
 			((struct jabber_data *)GJ_GC(gjc)->proto_data)->did_import = TRUE;
 		}
