@@ -254,9 +254,6 @@ extern struct prpl *proto_prpl[16];
 int serv_send_im(irc_t *irc, user_t *u, char *msg, int flags);
 int serv_send_chat(irc_t *irc, struct gaim_connection *gc, int id, char *msg );
 
-G_MODULE_EXPORT signed int do_iconv( char *from_cs, char *to_cs, char *src, char *dst, size_t size, size_t maxbuf );
-char *set_eval_charset( irc_t *irc, set_t *set, char *value );
-
 void nogaim_init();
 int proto_away( struct gaim_connection *gc, char *away );
 char *set_eval_away_devoice( irc_t *irc, set_t *set, char *value );
@@ -306,16 +303,6 @@ G_MODULE_EXPORT void serv_got_chat_in( struct gaim_connection *gc, int id, char
 G_MODULE_EXPORT void serv_got_chat_left( struct gaim_connection *gc, int id );
 /* void serv_finish_login( struct gaim_connection *gc ); */
 
-/* util.c */
-G_MODULE_EXPORT void strip_linefeed( gchar *text );
-G_MODULE_EXPORT char *add_cr( char *text );
-G_MODULE_EXPORT char *tobase64( const char *text );
-G_MODULE_EXPORT char *normalize( const char *s );
-G_MODULE_EXPORT time_t get_time( int year, int month, int day, int hour, int min, int sec );
-G_MODULE_EXPORT void strip_html( char *msg );
-G_MODULE_EXPORT char *escape_html( const char *html );
-G_MODULE_EXPORT void info_string_append(GString *str, char *newline, char *name, char *value);
-
 #ifdef WITH_MSN
 /* msn.c */
 G_MODULE_EXPORT void msn_init( struct prpl *ret );
@@ -342,5 +329,4 @@ G_MODULE_EXPORT void build_allow_list();
 
 struct conversation *conv_findchannel( char *channel );
 
-
 #endif
